use strict;
use feature qw(state);
use Benchmark qw(cmpthese);
use Types::Standard -types;
use Function::Parameters;
use Type::Params;
use Smart::Args::TypeTiny;
use Data::Validator;
use Params::ValidationCompiler qw(validation_for);

fun function_parameters_add(Int $x, Int $y) { return $x + $y }

sub type_params_add {
    state $check = compile(Int, Int);
    my ($x, $y) = $check->(@_);
    return $x + $y;
}

sub smart_args_typetiny_add {
    args_pos my $x => Int, my $y => Int;
    return $x + $y;
}

sub data_validator_add {
    state $v = Data::Validator->new( x => Int, y => Int )->with('SmartSequenced');
    my $args = $v->validate(@_);
    return $args->{x} + $args->{y};
}

sub params_validation_compiler_add {
    state $v = validation_for( params => [ { type => Int }, { type => Int } ] );
    my ($x, $y) = $v->(@_);
    return $x + $y;
}

my %cmp = (
  'Function::Parameters'
    => ['fp', sub { function_parameters_add(10, 10) }],
  'Type::Params'
    => ['tp', sub { type_params_add(10, 10) }],
  'Smart::Args::TypeTiny'
    => ['sa', sub { smart_args_typetiny_add(10, 10) }],
  'Data::Validator'
    => ['dv', sub { data_validator_add(10, 10) }],
  'Params::ValidationCompiler'
    => ['pv', sub {
            params_validation_compiler_add(10, 10) }],
);

print sprintf("perl version: %s\n", $]);
print "module/version/nick\n";
for (sort keys %cmp) {
    print sprintf("%s/%s/%s\n",
        $_, $_->VERSION, $cmp{$_}->[0])
}

cmpthese -1, {
    map { $cmp{$_}->[0] => $cmp{$_}->[1] } keys %cmp
};

# =>
# perl version: 5.028000
# 
# module/version/nick
#   Data::Validator/1.07/dv
#   Function::Parameters/2.001003/fp
#   Params::ValidationCompiler/0.30/pv
#   Smart::Args::TypeTiny/0.12/sa
#   Type::Params/1.004002/tp
# 
#        Rate   sa   dv   fp   pv   tp
# sa  73276/s   -- -46% -84% -88% -89%
# dv 135245/s  85%   -- -71% -78% -79%
# fp 472614/s 545% 249%   -- -23% -26%
# pv 614399/s 738% 354%  30%   --  -4%
# tp 639913/s 773% 373%  35%   4%   --
