use Test::More;
use Types::Standard -types;

# HashRef のキー、値が、Str、Int か
my $Map = Map[Str, Int];
ok $Map->check({ a => 1, b => 2 });
ok not $Map->check({ a => 1, b => 'aaa' });

# HashRef で`name`キーに対する値がStr、`age`キーに対する値がInt
my $Dict = Dict[name => Str, age => Int];
ok $Dict->check({ name => 'foo', age => 2 });
ok not $Dict->check({ name => 'bar', age => 'AA' });

# Optional で一部のキーがなくても良いようにもできる
{
    my $Dict = Dict[name => Str, id => Optional[Int]];
    ok $Dict->check({name => 'foo', id => 1});
    ok $Dict->check({name => 'bar'});
    ok not $Dict->check({name => 'bar', id => 'AAA'});
}

# ArrayRef で値がそれぞれ Str, Int
my $Tuple = Tuple[Str, Int];
ok $Tuple->check(['foo', 1]);
ok not $Tuple->check(['foo', 'aaa']);
ok not $Tuple->check(['foo', 1, 123]);

# Foo または Bar のインスタンスか否か
my $InstanceOf = InstanceOf['Foo', 'Bar'];
ok $InstanceOf->check(bless {}, 'Foo');
ok $InstanceOf->check(bless {}, 'Bar');
ok not $InstanceOf->check(bless {}, 'Baz');

# check, get_message メソッドを持つか
my $HasMethods = HasMethods['check', 'get_message'];
{
    use Type::Tiny;
    use Mouse::Meta::TypeConstraint;
    ok $HasMethods->check(Type::Tiny->new);
    ok $HasMethods->check(Mouse::Meta::TypeConstraint->new);
}

# Hoge,Fugaのうちのいずれか
my $Enum = Enum['Hoge','Fuga'];
ok $Enum->check('Hoge');
ok $Enum->check('Fuga');
ok not $Enum->check('Boo');

# 指定した演算子がOverloadされているか
my $Overload = Overload['&','|','~','>','<'];
ok $Overload->check(Type::Tiny->new);

# tie されているかいなか / 値に型制約をtieできる
tie my $tiestr, Str;
ok \$tiestr ~~ Tied;

$tiestr = 'hello';     # ok
eval { $tiestr = {} }; # die
ok $@;
done_testing;
